SystemOrganization addCategory: #'PharoBot-Core'!
SystemOrganization addCategory: #'PharoBot-Core-Tests'!

TestCase subclass: #BatteryTest
	instanceVariableNames: 'battery'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core-Tests'!

!BatteryTest class methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2012 18:41'!
packageNamesUnderTest

	^#('PharoBot-Core')! !

!BatteryTest methodsFor: 'percentage' stamp: 'NahuelGarbezza 12/9/2012 18:10'!
testChargePercentageLessThan100

	battery := Battery new.
	
	battery consume: 500.
	self assert: 50 equals: battery chargePercentage.
	battery consume: 200.
	self assert: 30 equals: battery chargePercentage.! !

!BatteryTest methodsFor: 'percentage' stamp: 'NahuelGarbezza 12/9/2012 18:11'!
testChargePercentageOf0

	battery := Battery new.
	
	battery consume: battery capacity.
	self assert: 0 equals: battery chargePercentage.! !

!BatteryTest methodsFor: 'percentage' stamp: 'NahuelGarbezza 12/9/2012 18:11'!
testChargePercentageOf100

	battery := Battery new.
	self assert: 100 equals: battery chargePercentage.! !

TestCase subclass: #RobotTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core-Tests'!

!RobotTest class methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2012 18:40'!
packageNamesUnderTest

	^#('PharoBot-Core')! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 18:36'!
testBrushDownIfItIsAlreadyBrushDownDoesNothing

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushDown.

	self shouldnt: [robot brushDown] raise: Error.! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 18:36'!
testBrushUpIfItIsAlreadyBrushUpDoesNothing

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushUp.

	self shouldnt: [robot brushUp] raise: Error.! !

!RobotTest methodsFor: 'directioning' stamp: 'NahuelGarbezza 12/22/2012 19:02'!
testDefaultDirectionIsNorth

	|robot|
	"when"
	robot := Robot new.
	self assert: robot direction equals: 90! !

!RobotTest methodsFor: 'positioning' stamp: 'NahuelGarbezza 12/22/2012 17:16'!
testDefaultPositionIsZeroZero

	|robot|
	"when"
	robot := Robot new.
	"then"
	self assert: robot position equals: 0@0! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 16:16'!
testDefaultStateIsBrushUp

	|robot|
	"when"
	robot := Robot new.
	"then"
	self assert: robot isBrushUp.! !

!RobotTest methodsFor: 'directioning' stamp: 'NahuelGarbezza 12/22/2012 18:18'!
testDirectioningToEast

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot east.
	"then"
	self assert: robot direction equals: 0! !

!RobotTest methodsFor: 'directioning' stamp: 'NahuelGarbezza 12/22/2012 18:19'!
testDirectioningToNorth

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot north.
	"then"
	self assert: robot direction equals: 90! !

!RobotTest methodsFor: 'directioning' stamp: 'NahuelGarbezza 12/22/2012 18:19'!
testDirectioningToSouth

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot south.
	"then"
	self assert: robot direction equals: 270! !

!RobotTest methodsFor: 'directioning' stamp: 'NahuelGarbezza 12/22/2012 18:18'!
testDirectioningToWest

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot west.
	"then"
	self assert: robot direction equals: 180! !

!RobotTest methodsFor: 'movements - brush down' stamp: 'NahuelGarbezza 12/22/2012 18:56'!
testLiftingTheBrushFinishesTheCurrentPath

	|robot|
	"given"
	robot := Robot new.
	robot brushDown.
	robot move: 40; north; move: 120.
	"when"
	robot brushUp.
	robot brushDown.
	"then"
	self assert: robot numberOfPaths equals: 2.
	self assert: robot paths first numberOfPoints equals: 3.
	self assert: robot paths second numberOfPoints equals: 1.! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:27'!
testMoveANegativeAmountInXAxis

	|robot|
	"given"
	robot := Robot new.
	robot east.
	"when"
	robot move: -70.
	"then"
	self assert: robot position equals: -70 @ 0! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:28'!
testMoveANegativeAmountInYAxis

	|robot|
	"given"
	robot := Robot new.
	robot south.
	"when"
	robot move: -20.
	"then"
	self assert: robot position equals: 0 @ 20! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:33'!
testMoveInDiagonal

	|robot|
	"given"
	robot := Robot new.
	robot direction: 45.
	"when"
	robot move: 8 sqrt.
	"then"
	self assert: robot position equals: 2 @ 2! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:26'!
testMoveStraightToEast

	|robot|
	"given"
	robot := Robot new.
	robot east.
	"when"
	robot move: 250.
	"then"
	self assert: robot position equals: 250 @ 0! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:26'!
testMoveStraightToNorth

	|robot|
	"given"
	robot := Robot new.
	robot north.
	"when"
	robot move: 30.
	"then"
	self assert: robot position equals: 0 @ 30! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:26'!
testMoveStraightToSouth

	|robot|
	"given"
	robot := Robot new.
	robot south.
	"when"
	robot move: 50.
	"then"
	self assert: robot position equals: 0 @ -50! !

!RobotTest methodsFor: 'movements' stamp: 'NahuelGarbezza 12/22/2012 18:26'!
testMoveStraightToWest

	|robot|
	"given"
	robot := Robot new.
	robot west.
	"when"
	robot move: 110.
	"then"
	self assert: robot position equals: -110 @ 0! !

!RobotTest methodsFor: 'movements - brush down' stamp: 'NahuelGarbezza 12/22/2012 19:02'!
testMovementsWithTheBrushDownModifyTheCurrentPath

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushDown; east.
	robot move: 30; south; move: 50.
	"then"
	self assert: robot currentPath numberOfPoints equals: 3.
	self assert: robot currentPath points first equals: 0 @ 0.
	self assert: robot currentPath points second equals: 30 @ 0.
	self assert: robot currentPath points third equals: 30 @ -50.! !

!RobotTest methodsFor: 'positioning' stamp: 'NahuelGarbezza 12/22/2012 18:16'!
testPositionCanBePassedInConstructor

	|robot|
	"when"
	robot := Robot newWithPosition: 100@50.
	"then"
	self assert: robot position equals: 100@50! !

!RobotTest methodsFor: 'printing' stamp: 'NahuelGarbezza 12/22/2012 19:07'!
testRobotStringRepresentation

	|robot robotString|
	"given"
	robot := Robot new.
	"when"
	robotString := robot printString.
	"then"
	self assert: robotString equals: 'Robot at (0@0), direction 90'.
	"when"
	robot move: -40; west; move: 50.
	robotString := robot printString.
	"then"
	self assert: robotString equals: 'Robot at (-50@ -40), direction 180'.
	! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 16:13'!
testStateIsSetToBrushDown

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushDown.
	"then"
	self assert: robot isBrushDown.
	self deny: robot isBrushUp.! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 16:14'!
testStateIsSetToBrushUp

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushUp.
	"then"
	self assert: robot isBrushUp.
	self deny: robot isBrushDown.! !

!RobotTest methodsFor: 'state' stamp: 'NahuelGarbezza 12/22/2012 18:43'!
testToggleBrushUpAndDown

	|robot|
	"given"
	robot := Robot new.
	"when"
	robot brushDown.
	"then"
	self assert: robot isBrushDown.
	"when"
	robot brushUp.
	"then"
	self assert: robot isBrushUp.
	"when"
	robot brushDown.
	"then"
	self assert: robot isBrushDown.! !

Error subclass: #EmptyBatteryError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!EmptyBatteryError commentStamp: '<historical>' prior: 0!
I represent the error situation when the robot tries to move without enough battery charge.!

!EmptyBatteryError class methodsFor: 'signaling' stamp: 'NahuelGarbezza 12/9/2012 18:32'!
signal

	^self signal: 'No hay suficiente bateria para moverse!!'! !

Object subclass: #Battery
	instanceVariableNames: 'charge capacity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!Battery commentStamp: '<historical>' prior: 0!
I represent the robot's battery. I have a capacity and a charge that decreases, and I can recharge myself.!

!Battery methodsFor: 'consume' stamp: 'NahuelGarbezza 12/9/2012 18:02'!
canConsume: amount

	^self charge >= amount! !

!Battery methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/9/2012 18:00'!
capacity
	^ capacity! !

!Battery methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:03'!
capacity: anObject
	capacity := anObject! !

!Battery methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/9/2012 17:59'!
charge
	^ charge! !

!Battery methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 17:59'!
charge: anObject
	charge := anObject! !

!Battery methodsFor: 'percentage' stamp: 'NahuelGarbezza 12/9/2012 18:03'!
chargePercentage

	^self charge * 100 / self capacity! !

!Battery methodsFor: 'consume' stamp: 'NahuelGarbezza 12/9/2012 18:01'!
consume: amount

	self charge: self charge - amount! !

!Battery methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:03'!
defaultCapacity

	^1000! !

!Battery methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 12/9/2012 18:03'!
initialize

	self capacity: self defaultCapacity.
	self recharge.
	^self! !

!Battery methodsFor: 'recharge' stamp: 'NahuelGarbezza 12/9/2012 18:00'!
recharge

	self charge: self capacity! !

Object subclass: #Robot
	instanceVariableNames: 'color battery currentPath paths direction state position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!Robot commentStamp: '<historical>' prior: 0!
I am a robot and I am able to do simple things like move through a certain space, leaving marks of the paths I am doing. I have a battery that consumes itself according to my moves, and can be recharged at every moment.!

!Robot class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 12/22/2012 18:16'!
newWithPosition: aPosition

	^self new position: aPosition; yourself! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:21'!
battery
	^ battery! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:21'!
battery: aBattery
	battery := aBattery! !

!Robot methodsFor: 'brush' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
brushDown

	self state brushDownTo: self! !

!Robot methodsFor: 'brush' stamp: 'NahuelGarbezza 12/9/2012 18:34'!
brushUp

	self state brushUpTo: self! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:30'!
cleanPaths

	self currentPath: (RobotPath new: self position).
	self paths: OrderedCollection new.
	self paths add: currentPath! !

!Robot methodsFor: 'private' stamp: 'FacundoMainere 4/5/2012 20:00'!
color
	^ color! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:23'!
color: aColor
	color := aColor! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:22'!
currentPath
	^ currentPath! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:22'!
currentPath: aPath
	currentPath := aPath! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
direction
	^ direction! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
direction: aDirection
	direction := aDirection! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:30'!
doMove: aDistance

	self position:
		(self position + (Point r: aDistance theta: Float pi / 180 * (self direction negated - 90))) rounded.! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
east

	self direction: 0! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:36'!
initialState

	^BrushUp new! !

!Robot methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
initialize

	self battery: Battery new.
	self color: Color random.
	self position: 0 @ 0.
	self state: self initialState.
	self direction: 90.
	self paths: OrderedCollection new.
	^self! !

!Robot methodsFor: 'brush' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isBrushDown

	^self state isDown! !

!Robot methodsFor: 'brush' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isBrushUp

	^self state isUp! !

!Robot methodsFor: 'positioning' stamp: 'NahuelGarbezza 12/9/2012 18:34'!
move: aDistance

	(self battery canConsume: aDistance)
		ifTrue: [
			self state move: aDistance toRobot: self.
			self battery consume: aDistance.]
		ifFalse: [ EmptyBatteryError signal ].! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
north

	self direction: 90! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/22/2012 18:55'!
numberOfPaths

	^self paths size! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:22'!
paths
	^ paths! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:23'!
paths: aCollectionOfPaths
	paths := aCollectionOfPaths! !

!Robot methodsFor: 'positioning' stamp: 'NahuelGarbezza 12/9/2012 18:30'!
position
	^ position! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:25'!
position: aPosition
	position := aPosition! !

!Robot methodsFor: 'printing' stamp: 'NahuelGarbezza 12/9/2012 18:39'!
printOn: aStream

	aStream
		nextPutAll: 'Robot at ';
		nextPutAll: self position printString;
		nextPutAll: ', direction ';
		nextPutAll: self direction printString.! !

!Robot methodsFor: 'battery' stamp: 'NahuelGarbezza 12/9/2012 18:26'!
refillBattery

	self battery recharge! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:29'!
south

	self direction: 270! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:30'!
startNewPath

	self currentPath: (RobotPath new: self position).
	self paths add: self currentPath! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:23'!
state
	^ state! !

!Robot methodsFor: 'private' stamp: 'NahuelGarbezza 12/9/2012 18:24'!
state: aState
	state := aState! !

!Robot methodsFor: 'direction' stamp: 'NahuelGarbezza 12/9/2012 18:28'!
west

	self direction: 180! !

Object subclass: #RobotPath
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!RobotPath commentStamp: '<historical>' prior: 0!
I am a path made by a robot. I can join infinite points that determine the route.!

!RobotPath class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 12/9/2012 18:14'!
new: aPoint

	^super new initialize: aPoint! !

!RobotPath methodsFor: 'path points' stamp: 'NahuelGarbezza 12/9/2012 18:15'!
addPoint: aPoint

	self points add: aPoint! !

!RobotPath methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 12/9/2012 18:15'!
initialize: aPoint

	self points: OrderedCollection new.
	self points add: aPoint.
	^self! !

!RobotPath methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2012 18:57'!
numberOfPoints

	^self points size! !

!RobotPath methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/9/2012 18:15'!
points
	^ points! !

!RobotPath methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/9/2012 18:15'!
points: anObject
	points := anObject! !

Object subclass: #RobotState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!RobotState commentStamp: '<historical>' prior: 0!
Represento un posible estado en el que se encuentre el robot del que formo parte.!

RobotState subclass: #BrushDown
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!BrushDown commentStamp: '<historical>' prior: 0!
I represent the state of a robot in which the brush is down.
!

!BrushDown methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:35'!
brushDownTo: aRobot

	"Nothing to do"! !

!BrushDown methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:37'!
brushUpTo: aRobot

	aRobot state: BrushUp new! !

!BrushDown methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isDown

	^true! !

!BrushDown methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isUp

	^false! !

!BrushDown methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:38'!
move: aDistance toRobot: aRobot
	
	aRobot doMove: aDistance.
	aRobot currentPath addPoint: aRobot position! !

!BrushDown methodsFor: 'printing' stamp: 'NahuelGarbezza 12/9/2012 18:38'!
printOn: aStream

	aStream << 'Brush down'! !

RobotState subclass: #BrushUp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!BrushUp commentStamp: '<historical>' prior: 0!
I represent the state of a robot in which the brush is up.
!

!BrushUp methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:37'!
brushDownTo: aRobot

	aRobot startNewPath.
	aRobot state: BrushDown new.! !

!BrushUp methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:37'!
brushUpTo: aRobot

	"Nothing to do"! !

!BrushUp methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isDown

	^false! !

!BrushUp methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isUp

	^true! !

!BrushUp methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:37'!
move: aDistance toRobot: aRobot
	
	aRobot doMove: aDistance! !

!BrushUp methodsFor: 'printing' stamp: 'NahuelGarbezza 12/9/2012 18:36'!
printOn: aStream

	aStream << 'Brush up'! !

!RobotState methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
brushDownTo: aRobot

	self subclassResponsibility
! !

!RobotState methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:35'!
brushUpTo: aRobot

	self subclassResponsibility
! !

!RobotState methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isDown

	self subclassResponsibility
! !

!RobotState methodsFor: 'testing' stamp: 'NahuelGarbezza 12/9/2012 18:33'!
isUp 

	self subclassResponsibility
! !

!RobotState methodsFor: 'actions' stamp: 'NahuelGarbezza 12/9/2012 18:34'!
move: aDistance toRobot: aRobot

	self subclassResponsibility! !

Object subclass: #RobotWorld
	instanceVariableNames: 'robots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PharoBot-Core'!

!RobotWorld commentStamp: '<historical>' prior: 0!
I contain all the robots of the system.!

!RobotWorld methodsFor: 'accessing' stamp: 'FacundoMainere 4/5/2012 19:56'!
robots
	^ robots! !

!RobotWorld methodsFor: 'accessing' stamp: 'FacundoMainere 4/5/2012 19:56'!
robots: anObject
	robots := anObject! !

!Point class methodsFor: '*pharobot-core' stamp: 'FacundoMainere 4/13/2012 13:02'!
r: t1 theta: t2
	^ (t1 * t2 sin) negated @  (t1 * t2 cos) negated! !
